/*
 * Timer/counter unit support for the JZ4780.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "tcu-jz4780.h"



// Channel abstraction.

Tcu_jz4780_channel::Tcu_jz4780_channel(l4_addr_t addr, uint8_t channel,
                                       l4_cap_idx_t irq)
: Tcu_channel(addr, channel, irq)
{
}



// Peripheral abstraction.

Tcu_jz4780_chip::Tcu_jz4780_chip(l4_addr_t start, l4_addr_t end)
: Tcu_chip(start, end)
{
}

Tcu_channel *Tcu_jz4780_chip::_get_channel(l4_addr_t addr, uint8_t channel,
                                           l4_cap_idx_t irq)
{
  return new Tcu_jz4780_channel(addr, channel, irq);
}



// C language interface functions.

void *jz4780_tcu_init(l4_addr_t tcu_base, l4_addr_t tcu_base_end)
{
  return (void *) new Tcu_jz4780_chip(tcu_base, tcu_base_end);
}

void *jz4780_tcu_get_channel(void *tcu, uint8_t channel, l4_cap_idx_t irq)
{
  return static_cast<Tcu_jz4780_chip *>(tcu)->get_channel(channel, irq);
}

void jz4780_tcu_disable(void *tcu_channel)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->disable();
}

void jz4780_tcu_enable(void *tcu_channel)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->enable();
}

int jz4780_tcu_is_enabled(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->is_enabled();
}

uint8_t jz4780_tcu_get_clock(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_clock();
}

void jz4780_tcu_set_clock(void *tcu_channel, uint8_t clock)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_clock(clock);
}

uint32_t jz4780_tcu_get_prescale(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_prescale();
}

void jz4780_tcu_set_prescale(void *tcu_channel, uint32_t prescale)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_prescale(prescale);
}

uint32_t jz4780_tcu_get_counter(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_counter();
}

void jz4780_tcu_set_counter(void *tcu_channel, uint32_t value)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_counter(value);
}

uint8_t jz4780_tcu_get_count_mode(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_count_mode();
}

void jz4780_tcu_set_count_mode(void *tcu_channel, uint8_t mode)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_count_mode(mode);
}

uint32_t jz4780_tcu_get_full_data_value(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_full_data_value();
}

void jz4780_tcu_set_full_data_value(void *tcu_channel, uint32_t value)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_full_data_value(value);
}

uint32_t jz4780_tcu_get_half_data_value(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_half_data_value();
}

void jz4780_tcu_set_half_data_value(void *tcu_channel, uint32_t value)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_full_data_value(value);
}

int jz4780_tcu_get_full_data_mask(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_full_data_mask();
}

void jz4780_tcu_set_full_data_mask(void *tcu_channel, int masked)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_full_data_mask(masked);
}

int jz4780_tcu_get_half_data_mask(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->get_half_data_mask();
}

void jz4780_tcu_set_half_data_mask(void *tcu_channel, int masked)
{
  static_cast<Tcu_jz4780_channel *>(tcu_channel)->set_half_data_mask(masked);
}

int jz4780_tcu_have_interrupt(void *tcu_channel)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->have_interrupt();
}

int jz4780_tcu_wait_for_irq(void *tcu_channel, unsigned int timeout)
{
  return static_cast<Tcu_jz4780_channel *>(tcu_channel)->wait_for_irq(timeout);
}
